/*
 * Copyright (C) 2008 Petr Hruska <phru@ucw.cz> 
 * 
 * This file is part of Cutter module.
 * 
 * Cutter module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#ifndef _BPARSER_H_
#define _BPARSER_H_

#include "../../str.h"

/* returns 0 if there is no method name present, or method keyword length. */
int bparser_method_length(char const* msg);

/* returns -1 on error */
int bparser_code(char const* msg);

/* msg is received message. param is an array with parameters like name="val".
 * Size of the array is determined by last item, which is null pointer. pos is
 * actual position in msg, which is reused by subsequent calls. It should be
 * position of first parameter for first call.
 *
 * the function parses a name="value" pair and searches for the name in params
 * array. Case is ignored during the search procedure. If a name is found, name
 * and val parameters are set. 
 *
 * parse returns 1 if a parameter was succesfully parsed, 0 if there was
 * nothing to parse (pos points to end of msg) and -1 on syntax error. 
 * 
 * If the name is found in params array, index parameter is set appropriately.
 * If the name is not found, index is set to -1. The function will return code
 * 1 indicating succesfull parsing even if the name of the parameter is not
 * found in params array.
 * 
 */
int bparser_parse(char* msg, char const* params[], str* name, str* val, 
int* pos, int* index);

void bparser_unescape(str* s);


#endif
