/*
 * Copyright (C) 2008 Petr Hruska <phru@ucw.cz> 
 * 
 * This file is part of Cutter module.
 * 
 * Cutter module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "../../sr_module.h"
#include "../tm/dlg.h"
#include "../tm/tm_load.h"

#include "cut.h"
#include "db_stuff.h"

struct tm_binds tmb;


static int cut_caller(struct dlg_full_t* info, rr_t* rr)
{
	str method = { "BYE", 3 };
	str body = { NULL, 0 };
	dlg_t d;
	rr_t *t, *s;
	str headers;
	int headers_malloced = 0;
	int n, cnt, rr_len, i;
	

	d.id.call_id = info->callid;
	d.id.loc_tag = info->to_tag;
	d.id.rem_tag = info->from_tag;

	d.loc_seq.value = info->to_cseq;
	d.loc_seq.is_set = info->to_cseq != -1;
	d.rem_seq.value = -1;
	d.rem_seq.is_set = 0;
	d.loc_uri = info->to_hfb;
	d.rem_uri = info->from_hfb;
	d.rem_target = info->from_contact;
	d.secure = 0;
	d.state = DLG_CONFIRMED;

	// we'll find top record route from initial invite
	cnt = 0; // number of records in route set
	for (t = rr; t; t = t->next) cnt++;

	n = cnt - info->n_hops; // number of records to skip
	if (n < 0) {
		LOG(L_ERR, "ERROR: cutter: cut_caller(): Invalid route set.\n");
		return -1;
	}
	for (t = rr; n > 0; t = t->next) n--;
	// t is top record from initial invite
	d.route_set = t;

	// setup extra headers including Record-Route
	if (t)
		rr_len = t->nameaddr.name.s - rr->nameaddr.name.s - 1; // we don't need last comma
	else
		rr_len = info->rr.len;
	if (rr_len > 0) {

		headers.len = strlen("Record-Route: ") + rr_len + strlen("\r\nMax-Forwards: 70\r\n");
		headers.s = pkg_malloc(headers.len);
		if (!headers.s) {
			LOG(L_ERR, "ERROR: cutter: cut_caller(): pkg_malloc failed.\n");
			return - 1;
		}
		headers_malloced = 1;

		i = strlen("Record-Route: ") + rr_len;
		memcpy(headers.s + i, "\r\nMax-Forwards: 70\r\n", strlen("\r\nMax-Forwards: 70\r\n"));
		s = rr;
		for (s = rr; s != t; s = s->next) {
			// print it
			i -= s->nameaddr.len;
			memcpy(headers.s + i, s->nameaddr.name.s, s->nameaddr.len);
			if (i > strlen("Record-Route: ")) headers.s[--i] = ',';
		}
		memcpy(headers.s, "Record-Route: ", strlen("Record-Route: "));
	} else {

		headers.s = "Max-Forwards: 70\r\n";
		headers.len = strlen("Max-Forwards: 70\r\n");
	}

	// send it to transaction module

	d.hooks.ru.s = NULL;
	d.hooks.ru.len = 0;
	d.hooks.nh.s = NULL;
	d.hooks.nh.len = 0;
	d.hooks.request_uri = NULL;
	d.hooks.next_hop = NULL;
	d.hooks.first_route = NULL;
	d.hooks.last_route = NULL;

	d.send_sock = NULL;

	if (tmb.t_request_within(
		&method, 
		&headers,
		&body,			// body
		&d,			// dialog structure
		NULL,			// transaction callback
		NULL			// callback pointer
	) < 0) {

		LOG(L_ERR, "ERROR: cutter: cut_caller(): sending request within dialog failed.\n");
		if (headers_malloced) pkg_free(headers.s);
		return -1;
	}

	if (headers_malloced) pkg_free(headers.s);
	return 0;
}


static void reverse_rr(rr_t** r)
{
	rr_t *t, *s, *u;

	s = NULL;	// empty list
	u = *r;		// full list
	while (u) {
		t = u;		
		u = u->next;	// remove first item
		t->next = s;	
		s = t;		// add it at beginnig of s
	}

	*r = s;
}


static int cut_callee(struct dlg_full_t* info, rr_t* rr)
{
	str method = { "BYE", 3 };
	str body = { NULL, 0 };
	dlg_t d;
	rr_t *t;
	str headers;
	int headers_malloced = 0;
	int n, rr_len, i;
	

	d.id.call_id = info->callid;
	d.id.loc_tag = info->from_tag;
	d.id.rem_tag = info->to_tag;

	d.loc_seq.value = info->from_cseq;
	d.loc_seq.is_set = info->from_cseq != -1;
	d.rem_seq.value = -1;
	d.rem_seq.is_set = 0;
	d.loc_uri = info->from_hfb;
	d.rem_uri = info->to_hfb;
	d.rem_target = info->to_contact;
	d.secure = 0;
	d.state = DLG_CONFIRMED;


	// we have to reverse order of rr
	reverse_rr(&rr);

	// we'll find first route inserted by our server and store it in t
	n = info->n_hops; // number of records to skip
	t = rr;
	for (t = rr; t && n > 0; t = t->next) n--;
	if (!t) {
		LOG(L_ERR, "ERROR: cutter: cut_caller(): Invalid route set.\n");
		return -1;
	}
	
	// Route set will include our route headers. It means that server will
	// send this packet to itself and relay it to uas. This allows script
	// to tear down rtp session.
	d.route_set = t;
	
	// setup extra headers including Record-Route
	rr_len = (t->nameaddr.name.s - info->rr.s) + t->nameaddr.name.len; // we don't need last comma
	DBG("DEBUG: cutter(): cut_callee(): rr_len = %d.\n", rr_len);
	if (rr_len > 0) {

		headers.len = strlen("Record-Route: ") + rr_len + strlen("\r\nMax-Forwards: 70\r\n");
		headers.s = pkg_malloc(headers.len);
		if (!headers.s) {
			LOG(L_ERR, "ERROR: cutter: cut_callee(): pkg_malloc failed.\n");
			reverse_rr(&rr);
			return -1;
		}
		headers_malloced = 1;

		memcpy(headers.s, "Record-Route: ", strlen("Record-Route: "));
		i = strlen("Record-Route: ");
		memcpy(headers.s + i, t->nameaddr.name.s, rr_len);
		i += rr_len;
		memcpy(headers.s + i, "\r\nMax-Forwards: 70\r\n", strlen("\r\nMax-Forwards: 70\r\n"));
	} else {
		headers.s = "Max-Forwards: 70\r\n";
		headers.len = strlen("Max-Forwards: 70\r\n");
	}

	// send it to transaction module

	d.hooks.ru.s = NULL;
	d.hooks.ru.len = 0;
	d.hooks.nh.s = NULL;
	d.hooks.nh.len = 0;
	d.hooks.request_uri = NULL;
	d.hooks.next_hop = NULL;
	d.hooks.first_route = NULL;
	d.hooks.last_route = NULL;

	d.send_sock = NULL;

	if (tmb.t_request_within(
		&method, 
		&headers,
		&body,			// body
		&d,			// dialog structure
		NULL,			// transaction callback
		NULL			// callback pointer
	) < 0) {

		LOG(L_ERR, "ERROR: cutter: cut_callee(): sending request within dialog failed.\n");
		if (headers_malloced) pkg_free(headers.s);
		reverse_rr(&rr);
		return -1;
	}

	if (headers_malloced) pkg_free(headers.s);
	reverse_rr(&rr);
	return 0;
}


void cutter_cut(struct dlg_full_t* info)
{
	rr_t* rr;
	
	DBG("DEBUG: cutter: cutter_cut(): Cutting dialog id = %d.\n", info->id);

	rr = NULL;
	if (parse_rr_body(info->rr.s, info->rr.len, &rr) < 0) {
		LOG(L_ERR, "ERROR: cutter: cutter_cut(): Can't parse route set.\n");
		return;
	}

	if (cut_caller(info, rr) < 0) {
		LOG(L_ERR, "ERROR: cutter: cutter_cut(): Error cutting caller.\n");
	}

	if (cut_callee(info, rr) < 0) {
		LOG(L_ERR, "ERROR: cutter: cutter_cut(): Error cutting caller.\n");
	}

	cutter_db_free_result(info);
	free_rr(&rr);
}

