/*
 * Copyright (C) 2008 Petr Hruska <phru@ucw.cz> 
 * 
 * This file is part of Cutter module.
 * 
 * Cutter module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include <stdlib.h>

#include "../../sr_module.h"
#include "../../db/db.h"

#include "db_stuff.h"

db_func_t cutter_dbf;
db_con_t* cutter_db_handle = 0;
char* cutter_db_table_name = "cutter_dlg";


// *
// ***** misc functions
// *


inline static void set_str_val(db_val_t* v, str s)
{
	if (s.s) {
		v->val.str_val = s;
		v->nul = 0;
	} else
		v->nul = 1;
}


static int cutter_find_dialog_general(str callid, str from_tag, str to_tag, 
	db_key_t* cols, int nc, db_key_t order, db_res_t** r)
{
	int n;

	static db_key_t keys[3] = {"sip_callid", "from_tag", "to_tag"};
	static db_op_t ops[3] = {OP_EQ, OP_EQ, OP_EQ};
	static db_val_t vals[3] = {
		{DB_STR, 0, {0}},
		{DB_STR, 0, {0}},
		{DB_STR, 0, {0}}
	};


	set_str_val(&vals[0], callid);
	set_str_val(&vals[1], from_tag);
	set_str_val(&vals[2], to_tag);

	if (to_tag.s) n = 3; else n = 2;

	if (cutter_dbf.query(cutter_db_handle, keys, ops, vals, cols, n, nc, order, r) < 0) {

		LOG(L_ERR, "ERROR: cutter: cutter_find_dialog_general(): db_query failed.\n");
		return -1;
	}

	return 0;
}


// PostgreSQL module returns int type columns like DB_INT, unfortunately
// MySQL module returns these columns as DB_STRING.
inline int static dbstr2int(db_val_t* v, int* n)
{
	char* endptr;

	if (v->type == DB_STRING) {
	
		if (!v->val.string_val[0]) {
			
			LOG(L_ERR, "ERROR: cutter: dbstr2int(): empty string.\n");
			*n = 0;
			return -1;
		}

		*n = strtol(v->val.string_val, &endptr, 10);
		if (*endptr) {
			LOG(L_ERR, "ERROR: cutter: dbstr2int(): nonumeric characters found.\n");
			*n = 0;
			return -1;
		}

		return 0;

	} else if (v->type == DB_INT) {
		*n = v->val.int_val;
		return 0;
	} else {

		LOG(L_ERR, "ERROR: cutter: dbstr2int(): unexpected type found.\n");
		*n = 0;
		return -1;
	}
}


// *
// ***** interface functions
// *

int cutter_db_dialog_callid2dlgid(str callid, int* id)
{
	int t;
	static db_key_t cols[1] = {"id"};
	db_res_t* r;

	static db_key_t keys[3] = {"sip_callid", "from_tag", "to_tag"};
	static db_op_t ops[3] = {OP_EQ, OP_NEQ, OP_NEQ};
	static db_val_t vals[3] = {
		{DB_STR, 0, {0}},
		{DB_STR, 0, {0}},
		{DB_STR, 0, {0}}
	};


	set_str_val(&vals[0], callid);
	vals[1].val.str_val.s = "";
	vals[1].val.str_val.len = 0;
	vals[1].nul = 0;
	vals[2].val.str_val.s = "";
	vals[2].val.str_val.len = 0;
	vals[2].nul = 0;

	set_str_val(&vals[0], callid);

	if (cutter_dbf.query(cutter_db_handle, keys, ops, vals, cols, 3, 1, NULL, &r) < 0) {

		LOG(L_ERR, "ERROR: cutter: cutter_find_dialog_id(): db_query failed.\n");
		return -1;
	}

	if (r->n > 0) {
		if (dbstr2int(&r->rows[0].values[0], id) < 0) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_id(): conversion of id to integer failed.\n");
			cutter_dbf.free_result(cutter_db_handle, r);
			return -1;
		}
		t = 0;
	} else
		t = -2;

	cutter_dbf.free_result(cutter_db_handle, r);
	return t;
}


int cutter_db_dialog_id(str callid, str from_tag, str to_tag, int* id)
{
	int t;
	static db_key_t cols[1] = {"id"};
	db_res_t* r;

	if (cutter_find_dialog_general(callid, from_tag, to_tag, cols, 1, NULL, &r) < 0) {

		LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_id(): db_query failed.\n");
		return -1;
	}

	if (r->n > 0) {
		if (dbstr2int(&r->rows[0].values[0], id) < 0) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_id(): conversion of id to integer failed.\n");
			cutter_dbf.free_result(cutter_db_handle, r);
			return -1;
		}
		t = 0;
	} else
		t = -2;

	cutter_dbf.free_result(cutter_db_handle, r);
	return t;
}


int cutter_db_dialog_short(str callid, str from_tag, str to_tag, struct dlg_short_t* info, int* direction)
{
	static db_key_t cols[4] = {"id", "check_time", "monitor", "S"};

	db_val_t* v;
	db_res_t* r;

	if (cutter_find_dialog_general(callid, from_tag, to_tag, cols, 4, NULL, &r) < 0) {

		LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_short(): db_query failed(1).\n");
		return -1;
	}

	if (r->n == 0) {
		//not found, let's check opposite direction

		cutter_dbf.free_result(cutter_db_handle, r);
		if (cutter_find_dialog_general(callid, to_tag, from_tag, cols, 4, NULL, &r) < 0) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_short(): db_query failed(2).\n");
			return -1;
		}
		if (direction) *direction = -1;
	} else {
		if (direction) *direction = 1;
	}

	if (r->n > 0) {
		v = r->rows[0].values;


		if (dbstr2int(&v[0], &info->id) < 0) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_short(): conversion of id from integer failed.\n");
			cutter_dbf.free_result(cutter_db_handle, r);
			return -1;
		}

		if (v[1].type != DB_DATETIME) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_short(): invalid type of check_time field.\n");
			cutter_dbf.free_result(cutter_db_handle, r);
			return -1;
		}
		info->check_time = v[1].val.time_val;

		if (dbstr2int(&v[2], &info->monitor) < 0) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_short(): conversion of monitor from integer failed.\n");
			cutter_dbf.free_result(cutter_db_handle, r);
			return -1;
		}

		if (dbstr2int(&v[3], &info->S) < 0) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_short(): conversion of S from integer failed.\n");
			cutter_dbf.free_result(cutter_db_handle, r);
			return -1;
		}

		cutter_dbf.free_result(cutter_db_handle, r);
		return 0;
	} else {
		cutter_dbf.free_result(cutter_db_handle, r);
		return -2;
	}
}


int cutter_db_dialog_full(int dlg_id, struct dlg_full_t* info, int* direction)
{
	static db_key_t keys[1] = {"id"};
	static db_op_t ops[1] = {OP_EQ};
	static db_val_t vals[1] = { {DB_INT, 0, {0}} };

	static db_key_t cols[18] = {
		"from_uri", "to_uri", "check_time", "monitor", 
		"sip_callid", "from_tag", "to_tag", "from_contact",
		"to_contact", "rr", "n_hops", "from_cseq",
		"to_cseq", "from_hfb", "to_hfb", "time_start",
		"time_finish", "S"};
	
	db_val_t* v;

	vals[0].val.int_val = dlg_id;

	if (cutter_dbf.query(cutter_db_handle, keys, ops, vals, cols, 1, 18, NULL, &info->r) < 0) {

		LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_full(): db_query failed.\n");
		return -1;
	}

	if (info->r->n > 0) {
		v = info->r->rows[0].values;

		if (	v[0].type != DB_STRING || v[1].type != DB_STRING  || v[2].type != DB_DATETIME ||
			v[4].type != DB_STRING || v[5].type != DB_STRING || v[6].type != DB_STRING ||
			v[7].type != DB_STRING || v[8].type != DB_STRING || v[9].type != DB_STRING ||
			v[13].type != DB_STRING || v[14].type != DB_STRING || v[15].type != DB_DATETIME ||
			v[16].type != DB_DATETIME) {
			
			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_full(): invalid type of result.\n");
			cutter_dbf.free_result(cutter_db_handle, info->r);
			info->r = NULL;
			return -1;
		}
		info->from_uri.s = (char*) v[0].val.string_val;
		info->from_uri.len = strlen(v[0].val.string_val);

		info->to_uri.s = (char*) v[1].val.string_val;
		info->to_uri.len = strlen(v[1].val.string_val);

		info->check_time = v[2].val.time_val;

		if (dbstr2int(&v[3], &info->monitor) < 0) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_full(): conversion of max_len from integer failed.\n");
			cutter_dbf.free_result(cutter_db_handle, info->r);
			info->r = NULL;
			return -1;
		}

		info->callid.s = (char*) v[4].val.string_val;
		info->callid.len = strlen(v[4].val.string_val);

		info->from_tag.s = (char*) v[5].val.string_val;
		info->from_tag.len = strlen(v[5].val.string_val);

		info->to_tag.s = (char*) v[6].val.string_val;
		info->to_tag.len = strlen(v[6].val.string_val);

		info->from_contact.s = (char*) v[7].val.string_val;
		info->from_contact.len = strlen(v[7].val.string_val);

		info->to_contact.s = (char*) v[8].val.string_val;
		info->to_contact.len = strlen(v[8].val.string_val);

		info->rr.s = (char*) v[9].val.string_val;
		info->rr.len = strlen(v[9].val.string_val);

		if (dbstr2int(&v[10], &info->n_hops) < 0) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_full(): conversion of n_hops from integer failed.\n");
			cutter_dbf.free_result(cutter_db_handle, info->r);
			info->r = NULL;
			return -1;
		}

		if (dbstr2int(&v[11], &info->from_cseq) < 0) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_full(): conversion of from_cseq from integer failed.\n");
			cutter_dbf.free_result(cutter_db_handle, info->r);
			info->r = NULL;
			return -1;
		}

		if (dbstr2int(&v[12], &info->to_cseq) < 0) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_full(): conversion of to_cseq from integer failed.\n");
			cutter_dbf.free_result(cutter_db_handle, info->r);
			info->r = NULL;
			return -1;
		}

		info->from_hfb.s = (char*) v[13].val.string_val;
		info->from_hfb.len = strlen(v[13].val.string_val);

		info->to_hfb.s = (char*) v[14].val.string_val;
		info->to_hfb.len = strlen(v[14].val.string_val);

		info->time_start = v[15].val.time_val;
		info->time_finish = v[16].val.time_val;

		if (dbstr2int(&v[17], &info->S) < 0) {

			LOG(L_ERR, "ERROR: cutter: cutter_db_dialog_full(): conversion of S from integer failed.\n");
			cutter_dbf.free_result(cutter_db_handle, info->r);
			info->r = NULL;
			return -1;
		}

		info->id = dlg_id;
		return 0;
	} else {
		cutter_dbf.free_result(cutter_db_handle, info->r);
		info->r = NULL;
		return -2;
	}
}


int cutter_db_free_result(struct dlg_full_t* info)
{
	int tmp;

	if (info->r) {
		tmp = cutter_dbf.free_result(cutter_db_handle, info->r);
		info->r = NULL;
		return tmp;
	}

	return 0;
}


int cutter_db_insert_dlg(struct dlg_full_t* info)
{
	int id;

	static db_key_t keys[18] = {
		"from_uri", "to_uri", "check_time", "monitor", 
		"sip_callid", "from_tag", "to_tag", "from_contact",
		"to_contact", "rr", "n_hops", "from_cseq",
		"to_cseq", "from_hfb", "to_hfb", "time_start",
		"time_finish", "S"};
	static db_val_t vals[18] = {
		{DB_STR, 0, {0} }, {DB_STR, 0, {0}}, {DB_DATETIME, 0, {0} }, {DB_INT, 0, {0}},
		{DB_STR, 0, {0} }, {DB_STR, 0, {0}}, {DB_STR, 0, {0}}, {DB_STR, 0, {0} },
		{DB_STR, 0, {0} }, {DB_STR, 0, {0}}, {DB_INT, 0, {0}}, {DB_INT, 0, {0}},
		{DB_INT, 0, {0}}, {DB_STR, 0, {0} }, {DB_STR, 0, {0} }, {DB_DATETIME, 0, {0} },
		{DB_DATETIME, 0, {0}}, {DB_INT, 0, {0}}
		};

	/* log packet to database */
	set_str_val(&vals[0], info->from_uri);
	set_str_val(&vals[1], info->to_uri);
	vals[2].val.time_val = info->check_time;
	vals[3].val.int_val = info->monitor;
	set_str_val(&vals[4], info->callid);
	set_str_val(&vals[5], info->from_tag);
	set_str_val(&vals[6], info->to_tag);
	set_str_val(&vals[7], info->from_contact);
	set_str_val(&vals[8], info->to_contact);
	set_str_val(&vals[9], info->rr);
	vals[10].val.int_val = info->n_hops;
	vals[11].val.int_val = info->from_cseq;
	vals[12].val.int_val = info->to_cseq;
	set_str_val(&vals[13], info->from_hfb);
	set_str_val(&vals[14], info->to_hfb);
	vals[15].val.time_val = info->time_start;
	vals[16].val.time_val = info->time_finish;
	vals[17].val.int_val = info->S;

	if (cutter_dbf.insert(cutter_db_handle, keys, vals, 18) < 0) {

		LOG(L_ERR, "ERROR: cutter: cutter_db_insert_dlg(): "
		"Error while inserting a dialog to database\n");
		return -1;
	}

	if (cutter_db_dialog_id(info->callid, info->from_tag, info->to_tag, &id) < 0) {

		LOG(L_ERR, "ERROR: cutter: cutter_db_insert_dlg(): "
		"Can't find just inserted record.\n");
		return -1;
	}
	
	return id;
}


int cutter_db_update_dlg(int id, int S, time_t time_start, time_t time_finish)
{
	static db_key_t keys[1] = {"id"};
	static db_op_t ops[1] = {OP_EQ};
	static db_val_t vals[1] = { {DB_INT, 0, {0}} };

	static db_key_t ukeys0[1] = {"S"};
	static db_key_t ukeys1[2] = {"S", "time_start"};
	static db_key_t ukeys2[2] = {"S", "time_finish"};
	static db_key_t ukeys3[3] = {"S", "time_start", "time_finish"};
	static db_val_t uvals[3] = { {DB_INT, 0, {0}}, {DB_DATETIME, 0, {0}}, {DB_DATETIME, 0, {0}} };

	db_key_t* ukeys;
	int un;

	DBG("DEBUG: cutter: cutter_db_update_dlg(): id = %d, S = %d, time_start = %d, time_finish = %d.\n", id, S, (int) time_start, (int) time_finish);

	if (!time_start && !time_finish) {
		un = 1;
		ukeys = ukeys0;
	} else if (!time_finish) {
		un = 2;
		ukeys = ukeys1;
		uvals[1].val.time_val = time_start;
	} else if (!time_start) {
		un = 2;
		ukeys = ukeys2;
		uvals[1].val.time_val = time_finish;
	} else {
		un = 3;
		ukeys = ukeys3;
		uvals[1].val.time_val = time_start;
		uvals[2].val.time_val = time_finish;
	}

	vals[0].val.int_val = id;
	uvals[0].val.int_val = S;

	if (cutter_dbf.update(cutter_db_handle, keys, ops, vals, ukeys, uvals, 1, un) < 0) {

		LOG(L_ERR, "ERROR: cutter: cutter_db_update_dlg(): db_query failed.\n");
		return -1;
	}

	return 0;
}


int cutter_db_update_rr(int id, str rr)
{
	static db_key_t keys[1] = {"id"};
	static db_op_t ops[1] = {OP_EQ};
	static db_val_t vals[1] = { {DB_INT, 0, {0}} };

	static db_key_t ukeys[1] = {"rr"};
	static db_val_t uvals[1] = { {DB_STR, 0, {0}} };

	if (rr.len > MAX_RR_LEN) {
		LOG(L_ERR, "ERROR: cutter: cutter_db_update_rr(): Maximum length of record-route header fields exceeded.\n");
		return -1;
	}

	vals[0].val.int_val = id;
	uvals[0].val.str_val = rr;

	if (cutter_dbf.update(cutter_db_handle, keys, ops, vals, ukeys, uvals, 1, 1) < 0) {

		LOG(L_ERR, "ERROR: cutter: cutter_db_update_rr(): db_query failed.\n");
		return -1;
	}

	return 0;
}

int cutter_db_update_cseq(int id, int cseq, int direction)
{
	static db_key_t keys[1] = {"id"};
	static db_op_t ops[1] = {OP_EQ};
	static db_val_t vals[1] = { {DB_INT, 0, {0}} };

	static db_key_t ukeys0[1] = {"from_cseq"};
	static db_key_t ukeys1[1] = {"to_cseq"};
	static db_val_t uvals[1] = { {DB_INT, 0, {0}} };

	static db_key_t* ukeys;

	switch (direction) {
	case -1: ukeys = ukeys1; break;
	case  1: ukeys = ukeys0; break;
	default:
		LOG(L_ERR, "ERROR: cutter: cutter_db_update_cseq(): Invalid direction.\n");
		return -1;
	}

	vals[0].val.int_val = id;
	uvals[0].val.int_val = cseq;

	if (cutter_dbf.update(cutter_db_handle, keys, ops, vals, ukeys, uvals, 1, 1) < 0) {

		LOG(L_ERR, "ERROR: cutter: cutter_db_update_cseq(): db_query failed.\n");
		return -1;
	}

	return 0;
}


int cutter_db_update_check(int id, time_t check_time, int monitor)
{
	static db_key_t keys[1] = {"id"};
	static db_op_t ops[1] = {OP_EQ};
	static db_val_t vals[1] = { {DB_INT, 0, {0}} };

	static db_key_t ukeys[2] = {"check_time", "monitor"};
	static db_val_t uvals[2] = { {DB_DATETIME, 0, {0}}, {DB_INT, 0, {0}} };

	vals[0].val.int_val = id;
	uvals[0].val.time_val = check_time;
	uvals[1].val.int_val = monitor;

	if (cutter_dbf.update(cutter_db_handle, keys, ops, vals, ukeys, uvals, 1, 2) < 0) {

		LOG(L_ERR, "ERROR: cutter: cutter_db_update_check(): db_query failed.\n");
		return -1;
	}

	return 0;
}
