/*
 * Copyright (C) 2008 Petr Hruska <phru@ucw.cz> 
 * 
 * This file is part of Cutter module.
 * 
 * Cutter module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#ifndef _DB_STUFF_H__
#define _DB_STUFF_H__

#include <time.h>

#include "../../str.h"
#include "../../db/db.h"

#define MAX_RR_LEN	2048	//database limit

extern db_con_t* cutter_db_handle;
extern db_func_t cutter_dbf;

// table names
extern char* cutter_db_table_name;

// some values from cutter_dlg table for common usage
struct dlg_short_t {
	int id;
	time_t check_time;
	int monitor;
	int S;
};

struct dlg_full_t {

	db_res_t *r; //if not null, call cutter_db_free_result()
	int id;
	str from_uri;	//really from uri from invite, used for accounting system
	str to_uri;	//request uri from invite
	time_t check_time;	// when do we have to ask a billing to continue the call or cut the call, 0 -> no ask, no cut
	int monitor;		// 1 -> ask a billing, 0 -> cut the call
	str callid;
	str from_tag;
	str to_tag;
	str from_contact;
	str to_contact;
	str rr;
	int n_hops;	// number of Record-route records of first invite (our position in rr-set)
	int from_cseq;
	int to_cseq;
	str from_hfb;	// from header field body 
	str to_hfb;
	time_t time_start;
	time_t time_finish;
	int S;
};


#define STATUS_START		0
#define STATUS_EARLY		1
#define STATUS_PENDING		2
#define STATUS_FAILED		3
#define STATUS_TERMINATED	4
#define STATUS_FORBIDDEN	5

/*

NONE 	=> START
	=> FORBIDDEN (no money)
START 	=> EARLY (1xx reply)
	=> PENDING (2xx reply)
	=> FAILED (3456 xx reply)
PENDING => TERMINATED


*/



/* Following functions are fetching informations about dialogs.
Return -1 on error, or -2 on not found. When record is successfully read from
database, 0 is returned.

According to RFC triplet callid, from_tag, to_tag identifies a dialog.
cutter_db_dialog_short() additionaly tries to swap from and to tags to find a
dialog when it was not found with original order. The direction can be NULL.
Otherwise it  is set to -1 if swapping was needed, or to +1 otherwise.
*/


int cutter_db_dialog_callid2dlgid(str callid, int* id);
int cutter_db_dialog_id(str callid, str from_tag, str to_tag, int* id);
int cutter_db_dialog_short(str callid, str from_tag, str to_tag, 
	struct dlg_short_t* info, int* direction);
int cutter_db_dialog_full(int dlg_id, 
	struct dlg_full_t* info, int* direction);

int cutter_db_free_result(struct dlg_full_t* info);

// Returns new dialog id on success, or -1 when failed.
int cutter_db_insert_dlg(struct dlg_full_t* info);

// Does not setup time with zero value.
int cutter_db_update_dlg(int id, int S, time_t time_start, time_t time_finish);

int cutter_db_update_rr(int id, str rr);
int cutter_db_update_cseq(int id, int cseq, int direction);
int cutter_db_update_check(int id, time_t check_time, int monitor);

#endif
