/*
 * Copyright (C) 2008 Petr Hruska <phru@ucw.cz> 
 * 
 * This file is part of Cutter module.
 * 
 * Cutter module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "../../fifo_server.h"
#include "../../sr_module.h"

#include "fifo.h"
#include "cut.h"
#include "timer.h"
#include "db_stuff.h"

#define MAX_DATA_LEN 512

static void cut_call(str callid)
{
	struct dlg_full_t info;
	struct dlg_short_t si;
	int id;
	
	if (cutter_db_dialog_callid2dlgid(callid, &id) < 0) {

		LOG(L_ERR, "ERROR: cutter: cut_call(): Can't find dialog in a database.\n");
		return;
	}
	
	DBG("DEBUG: cutter: cut_call(): dialog id = %d.\n", id);

	if (cutter_db_dialog_full(id, &info, NULL) < 0) {
		LOG(L_ERR, "ERROR: cutter: cut_call(): Can't fetch data from a database.\n");
		return;
	}

	cutter_cut(&info);

	int scheduled = (info.S == STATUS_PENDING) && info.check_time;
	if (scheduled) {
		si.id = info.id;
		si.check_time = info.check_time;
		si.monitor = info.monitor;
		si.S = info.S;

		if (cutter_unschedule(&si) < 0) {
			LOG(L_ERR, "ERROR: cutter: call_cut(): "
			"could not remove scheduled call check.\n");
		}
	}

	cutter_db_free_result(&info);
}


static int cut_fifo_cmd( FILE *fifo_stream, char* response_file)
{
	static char data[MAX_DATA_LEN];
	static char callid_data[MAX_DATA_LEN];
	int data_len;
	time_t time;
	str callid;

	
	if (response_file==0 || *response_file==0 ) { 

		LOG(L_ERR, "ERROR: cutter: cut_fifo_cmd(): null file\n");
		return -1;
	}

	if (!read_line(data, MAX_DATA_LEN, fifo_stream, &data_len)) {

		fifo_reply(response_file, "500 too long data");
		return -1;
	}

	if (response_file) {
		
		DBG("DEBUG: cutter: cut_fifo_cmd(): %s\n", data);
		fifo_reply(response_file, "200 ok\ncutting call\n");

		callid.s = callid_data;

		int r = sscanf(data, "%s %ld", callid.s, &time);
		if (r != 2) {
			
			LOG(L_ERR, "ERROR: cutter: cut_fifo_cmd(): parsing error\n");
			return -1;
		}

		callid.len = strlen(callid.s);
		cut_call(callid);
	}

	return 1;
}


int cutter_fifo_init()
{
	if (register_fifo_cmd(cut_fifo_cmd, "cutter_cut", NULL) < 0) {
		LOG(L_ERR, "ERROR: unable to register 'cutter_cut' FIFO cmd\n");
		return -1;
	}

	return 0;
}
