/*
 * Copyright (C) 2008 Petr Hruska <phru@ucw.cz> 
 * 
 * This file is part of Cutter module.
 * 
 * Cutter module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include <sys/un.h>
#include <unistd.h>
#include <errno.h>


#include "rtpproxy.h"

#define RTPP_SOCKET	"/tmp/rtpproxy.sock"


int rtpp_send(struct iovec* v, int vcnt)
{
	struct sockaddr_un addr;
	int fd, len;
	static char buf[256];


	memset(&addr, 0, sizeof(addr));
	addr.sun_family = AF_UNIX;
	strcpy(addr.sun_path, RTPP_SOCKET);

	fd = socket(AF_LOCAL, SOCK_STREAM, 0);
	if (fd < 0) {
	       	LOG(L_ERR, "ERROR: send_rtpp_command: can't create socket\n");
	       	return -1;
       	}
       	if (connect(fd, (struct sockaddr *) &addr, sizeof(addr)) < 0) {

	       	close(fd);
		LOG(L_ERR, "ERROR: cutter: rtpp_send(): can't connect to RTP proxy\n");
		return -1;
       	}

       	do {
	       	len = writev(fd, v, vcnt);
       	} while (len == -1 && errno == EINTR);

       	if (len <= 0) {
	       	close(fd);
		LOG(L_ERR, "ERROR: cutter: rtpp_send(): can't send command to a RTP proxy\n");
	       	return -1;
       	}

       	do {
	       	len = read(fd, buf, sizeof(buf) - 1);
       	} while (len == -1 && errno == EINTR);

       	close(fd);
       	if (len <= 0) {
		LOG(L_ERR, "ERROR: cutter: rtpp_send(): can't read reply from a RTP proxy\n");
	       	return -1;
       	}

	DBG("DEBUG: cutter: rtpp_send(): read reply '\%.*s'\n", len, buf);
	
	if (len != 2) return -1;
	if (memcmp(buf, "0\n", 2)) return -1;

	return 0;
}

int rtpp_start_timer(str callid, str ftag, str totag)
{
	struct iovec v[6] = {
		{"C ", 2},
		{callid.s, callid.len},
		{" ", 1},
		{ftag.s, ftag.len},
		{" ", 1},
		{totag.s, totag.len}
	};

	return rtpp_send(v, 6);
}
