/*
 * Copyright (C) 2008 Petr Hruska <phru@ucw.cz> 
 * 
 * This file is part of Cutter module.
 * 
 * Cutter module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include <time.h>

#include "../../sr_module.h"
#include "../../timer.h"

#include "timer.h"
#include "timer_data.h"
#include "db_stuff.h"
#include "cut.h"
#include "billing.h"
#include "cutter.h"


static int check_continue(struct dlg_full_t * info)
{
	int result;
	struct dlg_short_t si;

	si.id = info->id;
	si.S = info->S;
	// default values
	si.check_time = 0;
	si.monitor = 0;

	if (billing_continue(bbuf, &info->callid, &si.check_time, &si.monitor, &result) < 0) {
		LOG(L_ERR, "ERROR: cutter: check_continue(): Billing error, cutting call.\n");
		cutter_cut(info);
		return -1;
	}

	if (result / 100 != 2) {
		DBG("DEBUG: cutter: check_continue(): Billing returned non-2xx code, cutting call.\n");
		cutter_cut(info);
		return 0;
	}

	if (cutter_db_update_check(si.id, si.check_time, si.monitor) < 0) {
		LOG(L_ERR, "ERROR: cutter: check_continue(): "
		"could not update checking info in database.\n");
		cutter_cut(info);
		return 0;
	}

	if (si.check_time > 0 && si.monitor >= 0) {

		if (cutter_schedule(&si, si.monitor > 0 ? -billing_advance : 0) < 0) {
			LOG(L_ERR, "ERROR: cutter: check_continue(): Can't schedule call.\n");
			cutter_cut(info);
			return -1;
		}
		DBG("DEBUG: cutter: check_continue(): Call scheduled to %ld.\n", si.check_time);
	}

	return 0;

}

// ask billing, or cut
static void process_call(struct timer_item_t* ti)
{
	struct dlg_full_t info;
	
	DBG("DEBUG: cutter: process_call(): dialog id = %d.\n", ti->dlg);

	if (cutter_db_dialog_full(ti->dlg, &info, NULL) < 0) {
		LOG(L_ERR, "ERROR: cutter: process_call(): Can't fetch data from a database.\n");
		return;
	}

	DBG("DEBUG: cutter: process_call(): monitor = %d.\n", info.monitor);

	if (!info.check_time) {
		LOG(L_ERR, "ERROR: cutter: process_call(): Call with zero check_time found.\n");
		cutter_db_free_result(&info);
		return;
	}

	if (!info.monitor) {
		DBG("DEBUG: cutter: process_call(): monitor=0 -> cutting call.\n");
		cutter_cut(&info);
	} else
		check_continue(&info);

	cutter_db_free_result(&info);
}

static void ticker(unsigned int ticks, void* param)
{
	time_t t;
	int r;
	struct timer_item_t ti;

	t = time(NULL);

	r = td_extract_elapsed(&ti, t);
	while (!r) {
		DBG("DEBUG: cutter: ticker(): found something\n");
		
		process_call(&ti);

		r = td_extract_elapsed(&ti, t);
	}

	if (r == -1) {
		LOG(L_ERR, "ERROR: cutter: ticker(): "
		"could not fetch timer event.\n");
		return;
	}
}

int cutter_timer_init()
{
	if (td_init() < 0) {
		LOG(L_ERR, "ERROR: cutter: cutter_timer_init(): "
		"Unable to initialize timer data.\n");
		return -1;
	};

	if (register_timer(ticker, NULL, 1) < 0) {
		LOG(L_ERR, "ERROR: cutter: cutter_timer_init(): "
		"Unable to register timer.\n");
		return -1;
	}

	return 0;
}


int cutter_schedule(struct dlg_short_t* si, int shift)
{
	struct timer_item_t ti;

	if (!si->check_time) {
		LOG(L_ERR, "ERROR: cutter: cutter_schedule(): "
		"Invalid check_time.\n");
		return -1;
	}

	DBG("DEBUG: cutter: cutter_schedule(): "
	"scheduling cut of dialog %d %ld seconds into future.\n", 
	si->id, si->check_time + shift - time(NULL));

	ti.dlg = si->id;
	ti.t = si->check_time + shift;

	if (td_insert(&ti) < 0) {
		LOG(L_ERR, "ERROR: cutter: cutter_schedule(): "
		"could not schedule timer event.\n");
		return -1;
	}

	return 0;
}


int cutter_unschedule(struct dlg_short_t* si)
{
	if (td_remove(si->id) < 0) {
		LOG(L_ERR, "ERROR: cutter: cutter_unschedule(): "
		"could not unschedule timer event (id = %d).\n", si->id);
		return -1;
	}

	return 0;
}
